@@ -22,6 +22,7 @@
 #include <libglom/data_structure/layout/layoutitem_field.h>
 #include <libglom/data_structure/layout/layoutitem_portal.h>
 #include <glibmm/i18n.h>
+#include <iostream> 
 
 namespace Glom
 {
@@ -110,6 +111,35 @@ bool LayoutGroup::has_field(const Glib::ustring& field_name) const
   return false;
 }
 
+bool LayoutGroup::has_field(const Glib::ustring& parent_table_name, const Glib::ustring& table_name, const Glib::ustring& field_name) const
+{
+  for(type_list_items::const_iterator iter = m_list_items.begin(); iter != m_list_items.end(); ++iter)
+  {
+    sharedptr<LayoutItem> item = *iter;
+    sharedptr<LayoutItem_Field> field_item = sharedptr<LayoutItem_Field>::cast_dynamic(item);
+    if(field_item)
+    {
+      if( (field_item->get_name() == field_name) &&
+        (field_item->get_table_used(parent_table_name) == table_name))
+      {
+        return true;
+      }
+    }
+    else
+    {
+      //Recurse into the child groups:
+      sharedptr<LayoutGroup> group_item = sharedptr<LayoutGroup>::cast_dynamic(item);
+      if(group_item)
+      {
+        if(group_item->has_field(parent_table_name, table_name, field_name))
+          return true;
+      }
+    }
+  }
+
+  return false;
+}
+
 bool LayoutGroup::has_any_fields() const
 {
   for(type_list_items::const_iterator iter = m_list_items.begin(); iter != m_list_items.end(); ++iter)
@@ -186,6 +216,7 @@ LayoutGroup::type_list_const_items LayoutGroup::get_items_recursive() const
   for(type_list_items::const_iterator iter = m_list_items.begin(); iter != m_list_items.end(); ++iter)
   {
     const sharedptr<const LayoutItem> item = *iter;
+    
     sharedptr<const LayoutGroup> group = sharedptr<const LayoutGroup>::cast_dynamic(item);
     if(group)
     {
@@ -206,6 +237,7 @@ LayoutGroup::type_list_items LayoutGroup::get_items_recursive()
   for(type_list_items::const_iterator iter = m_list_items.begin(); iter != m_list_items.end(); ++iter)
   {
     const sharedptr<LayoutItem> item = *iter;
+    
     sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::cast_dynamic(item);
     if(group)
     {
@@ -219,6 +251,28 @@ LayoutGroup::type_list_items LayoutGroup::get_items_recursive()
   return result;
 }
 
+LayoutGroup::type_list_const_items LayoutGroup::get_items_recursive_with_groups() const
+{
+  type_list_const_items result;
+
+  for(type_list_items::const_iterator iter = m_list_items.begin(); iter != m_list_items.end(); ++iter)
+  {
+    const sharedptr<const LayoutItem> item = *iter;
+    
+    //Add the item itself:
+    result.push_back(item);
+    
+    sharedptr<const LayoutGroup> group = sharedptr<const LayoutGroup>::cast_dynamic(item);
+    if(group)
+    {
+      const type_list_const_items sub_result = group->get_items_recursive();
+      result.insert(result.end(), sub_result.begin(), sub_result.end());
+    }
+  }
+
+  return result;
+}
+
 void LayoutGroup::remove_relationship(const sharedptr<const Relationship>& relationship)
 {
   LayoutGroup::type_list_items::iterator iterItem = m_list_items.begin();
@@ -313,6 +367,37 @@ void LayoutGroup::remove_field(const Glib::ustring& table_name, const Glib::ustr
   }
 }
 
+void LayoutGroup::remove_field(const Glib::ustring& parent_table_name, const Glib::ustring& table_name, const Glib::ustring& field_name)
+{
+  //Look at each item:
+  LayoutGroup::type_list_items::iterator iterItem = m_list_items.begin();
+  while(iterItem != m_list_items.end())
+  {
+    sharedptr<LayoutItem> item = *iterItem;
+    sharedptr<LayoutItem_Field> field_item = sharedptr<LayoutItem_Field>::cast_dynamic(item);
+    if(field_item)
+    {
+      if(field_item->get_table_used(parent_table_name) == table_name)
+      {
+        if(field_item->get_name() == field_name)
+        {
+          m_list_items.erase(iterItem);
+          iterItem = m_list_items.begin(); //Start again, because we changed the container.AddDel
+          continue;
+        }
+      }
+    }
+    else
+    {
+      sharedptr<LayoutGroup> sub_group = sharedptr<LayoutGroup>::cast_dynamic(item);
+      if(sub_group)
+        sub_group->remove_field(parent_table_name, table_name, field_name);
+    }
+
+    ++iterItem;
+  }
+}
+
 void LayoutGroup::change_related_field_item_name(const Glib::ustring& table_name, const Glib::ustring& field_name, const Glib::ustring& field_name_new)
 {
   //Look at each item:
@@ -350,7 +435,10 @@ void LayoutGroup::change_field_item_name(const Glib::ustring& table_name, const
   for(LayoutGroup::type_list_items::iterator iterItem = m_list_items.begin(); iterItem != m_list_items.end(); ++iterItem)
   {
     sharedptr<LayoutItem> item = *iterItem;
-    sharedptr<LayoutItem_Field> field_item = sharedptr<LayoutItem_Field>::cast_dynamic(item);
+    sharedptr<LayoutItem_Field> field_item = 
+      sharedptr<LayoutItem_Field>::cast_dynamic(item);
+    
+    //Field layout items:
     if(field_item)
     {
       if(field_item->get_has_relationship_name()) //If it's a related table (this would be a self-relationship)
@@ -373,6 +461,16 @@ void LayoutGroup::change_field_item_name(const Glib::ustring& table_name, const
     }
     else
     {
+      //Formatting:
+      sharedptr<LayoutItem_WithFormatting> with_formatting = 
+        sharedptr<LayoutItem_WithFormatting>::cast_dynamic(item);
+      if(with_formatting)
+      {
+        FieldFormatting& formatting = with_formatting->m_formatting;
+        formatting.change_field_item_name(table_name, field_name, field_name_new);
+      }
+   
+      //Recurse into sub-groups:
       sharedptr<LayoutGroup> sub_group = sharedptr<LayoutGroup>::cast_dynamic(item);
       if(sub_group)
         sub_group->change_field_item_name(table_name, field_name, field_name_new);
@@ -412,6 +510,7 @@ void LayoutGroup::change_relationship_name(const Glib::ustring& table_name, cons
 
 Glib::ustring LayoutGroup::get_part_type_name() const
 {
+  //Translators: This is the name of a UI element (a layout part name).
   return _("Group");
 }
 
