@@ -44,12 +44,26 @@ Dialog_Layout_List_Related::Dialog_Layout_List_Related(BaseObjectType* cobject,
   m_radio_navigation_none(0),
   m_radio_navigation_specify(0),
   m_label_navigation_automatic(0),
-  m_combo_navigation_specify(0)
-{
+  m_combo_navigation_specify(0),
+  m_spinbutton_row_line_width(0),
+  m_spinbutton_column_line_width(0),
+  m_colorbutton_line(0),
+  m_for_print_layout(false)
+{  
   // Show the appropriate alternate widgets:
   m_box_table_widgets->hide();
   m_box_related_table_widgets->show();
   m_box_related_navigation->show();
+  m_hbox_rows_count->show();
+  
+  m_spinbutton_rows_count_min->set_range(0, 100); //Otherwise only 0 would be allowed.
+  m_spinbutton_rows_count_min->set_increments(1, 10); //Otherwise the buttons do nothing.
+  m_spinbutton_rows_count_min->signal_value_changed().connect(
+    sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+  m_spinbutton_rows_count_max->set_range(0, 100); //Otherwise only 0 would be allowed.
+  m_spinbutton_rows_count_max->set_increments(1, 10); //Otherwise the buttons do nothing.
+  m_spinbutton_rows_count_max->signal_value_changed().connect(
+    sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
 
   builder->get_widget_derived("combo_relationship_name", m_combo_relationship);
   m_combo_relationship->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_combo_relationship_changed));
@@ -57,7 +71,7 @@ Dialog_Layout_List_Related::Dialog_Layout_List_Related(BaseObjectType* cobject,
   builder->get_widget("checkbutton_show_child_relationships", m_checkbutton_show_child_relationships);
   m_checkbutton_show_child_relationships->signal_toggled().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_checkbutton_show_child_relationships));
 
-  
+
 
   builder->get_widget("radiobutton_navigation_automatic", m_radio_navigation_automatic);
   builder->get_widget("label_navigation_automatic", m_label_navigation_automatic);
@@ -70,7 +84,17 @@ Dialog_Layout_List_Related::Dialog_Layout_List_Related(BaseObjectType* cobject,
   make_sensitivity_depend_on_toggle_button(*m_radio_navigation_specify, *m_combo_navigation_specify);
   m_combo_navigation_specify->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_combo_navigation_specific_changed));
 
- 
+  builder->get_widget("spinbutton_row_line_width", m_spinbutton_row_line_width);
+  m_spinbutton_row_line_width->signal_value_changed().connect(
+    sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+  builder->get_widget("spinbutton_column_line_width", m_spinbutton_column_line_width);
+  m_spinbutton_column_line_width->signal_value_changed().connect(
+    sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+  builder->get_widget("colorbutton_line", m_colorbutton_line);
+  m_colorbutton_line->signal_color_set().connect(
+    sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+
+
   m_modified = false;
 
   //show_all_children();
@@ -89,8 +113,10 @@ Dialog_Layout_List_Related::~Dialog_Layout_List_Related()
 }
 
 
-void Dialog_Layout_List_Related::set_document(const Glib::ustring& layout_name, const Glib::ustring& layout_platform, Document* document, const sharedptr<const LayoutItem_Portal>& portal, const Glib::ustring& from_table)
+void Dialog_Layout_List_Related::set_document(const Glib::ustring& layout_name, const Glib::ustring& layout_platform, Document* document, const sharedptr<const LayoutItem_Portal>& portal, const Glib::ustring& from_table, bool for_print_layout)
 {
+  m_for_print_layout = for_print_layout;
+
   //Ignore the provided from_table if the portal has one:
   Glib::ustring actual_from_table;
   if(portal)
@@ -99,7 +125,7 @@ void Dialog_Layout_List_Related::set_document(const Glib::ustring& layout_name,
     if(portal_from_table.empty())
       actual_from_table = portal_from_table;
   }
-  
+
   if(actual_from_table.empty())
     actual_from_table = from_table;
 
@@ -107,11 +133,35 @@ void Dialog_Layout_List_Related::set_document(const Glib::ustring& layout_name,
     m_portal = glom_sharedptr_clone(portal);
   else
     m_portal = sharedptr<LayoutItem_Portal>::create(); //The rest of the class assumes that this is not null.
-  
-  type_vecLayoutFields empty_fields; //Just to satisfy the base class.
+
+  gulong rows_count_min = 0;
+  gulong rows_count_max = 0;
+  m_portal->get_rows_count(rows_count_min, rows_count_max);
+  m_spinbutton_rows_count_min->set_value(rows_count_min);
+  m_spinbutton_rows_count_max->set_value(rows_count_max);
+
+  type_vecConstLayoutFields empty_fields; //Just to satisfy the base class.
   Dialog_Layout::set_document(layout_name, layout_platform, document, actual_from_table, empty_fields);
   //m_table_name is now actually the parent_table_name.
 
+  //Hide unwanted widgets and show extra ones:
+  if(for_print_layout)
+  {
+    m_box_related_navigation->hide();
+    m_box_frame_lines->show();
+    
+    m_spinbutton_row_line_width->set_value(
+      portal->get_print_layout_row_line_width());
+    m_spinbutton_column_line_width->set_value(
+      portal->get_print_layout_column_line_width());
+
+    const Gdk::RGBA color( portal->get_print_layout_line_color() );
+    m_colorbutton_line->set_rgba(color);
+
+    //Avoid showing formatting options that are about editing:
+    m_editable_layout = false;
+  }
+
   update_ui();
 }
 
@@ -132,7 +182,7 @@ void Dialog_Layout_List_Related::update_ui(bool including_relationship_list)
     if(including_relationship_list)
     {
       bool show_child_relationships = m_checkbutton_show_child_relationships->get_active();
-        
+
       //For the showing of child relationships if necessary:
       if(!show_child_relationships && m_portal->get_related_relationship())
       {
@@ -158,7 +208,7 @@ void Dialog_Layout_List_Related::update_ui(bool including_relationship_list)
     Document::type_list_layout_groups mapGroups;
     if(m_portal)
     {
-      m_combo_relationship->set_selected_relationship(m_portal->get_relationship(), m_portal->get_related_relationship()); 
+      m_combo_relationship->set_selected_relationship(m_portal->get_relationship(), m_portal->get_related_relationship());
 
       mapGroups.push_back(m_portal);
       document->fill_layout_field_details(related_table_name, mapGroups); //Update with full field information.
@@ -229,22 +279,13 @@ void Dialog_Layout_List_Related::update_ui(bool including_relationship_list)
   }
 
   //Describe the automatic navigation:
-  sharedptr<const UsesRelationship> relationship_navigation_automatic;
-  bool navigation_automatic_main = false;
-  relationship_navigation_automatic = get_portal_navigation_relationship_automatic(m_portal, navigation_automatic_main);
-  Glib::ustring automatic_navigation_description;
-  
-  if(navigation_automatic_main)
-    automatic_navigation_description = m_portal->get_relationship_name_used();
-  else if(relationship_navigation_automatic)
+  sharedptr<const UsesRelationship> relationship_navigation_automatic = 
+    m_portal->get_portal_navigation_relationship_automatic(document);
+  Glib::ustring automatic_navigation_description = 
+    m_portal->get_relationship_name_used(); //TODO: Use get_relationship_display_name() instead?
+  if(relationship_navigation_automatic) //This is a relationship in the related table.
   {
-    automatic_navigation_description = m_portal->get_relationship_name_used();
-
-    if(relationship_navigation_automatic->get_has_relationship_name())
-      automatic_navigation_description += ("::" + relationship_navigation_automatic->get_relationship_name());
-
-    if(relationship_navigation_automatic->get_has_related_relationship_name())
-      automatic_navigation_description += ("::" + relationship_navigation_automatic->get_related_relationship_name());
+    automatic_navigation_description += ("::" + relationship_navigation_automatic->get_relationship_display_name());
   }
 
   if(automatic_navigation_description.empty())
@@ -264,7 +305,7 @@ void Dialog_Layout_List_Related::save_to_document()
   if(m_modified)
   {
     //Get the data from the TreeView and store it in the document:
-
+    
     //Get the groups and their fields:
     Document::type_list_layout_groups mapGroups;
 
@@ -320,7 +361,20 @@ void Dialog_Layout_List_Related::save_to_document()
       uses_rel->set_related_relationship(sharedptr<Relationship>());
       m_portal->set_navigation_type(LayoutItem_Portal::NAVIGATION_NONE);
     }
-
+    
+    m_portal->set_rows_count(
+      m_spinbutton_rows_count_min->get_value(),
+      m_spinbutton_rows_count_max->get_value());
+    
+    if(m_for_print_layout)
+    {
+      m_portal->set_print_layout_row_line_width(
+        m_spinbutton_row_line_width->get_value());
+      m_portal->set_print_layout_column_line_width(
+        m_spinbutton_column_line_width->get_value());      
+      m_portal->set_print_layout_line_color(
+        m_colorbutton_line->get_rgba().to_string() );
+    }
   }
 }
 
@@ -338,45 +392,45 @@ void Dialog_Layout_List_Related::on_combo_relationship_changed()
   sharedptr<Relationship> relationship = m_combo_relationship->get_selected_relationship(relationship_related);
   if(!relationship)
     return;
-    
+
   //Check that the relationship is appropriate for use in a related records portal.
-  //The relationship's to field may not be a unique field, because that would 
+  //The relationship's to field may not be a unique field, because that would
   //prevent the portal from having multiple records.
-  sharedptr<Field> to_key_field = 
-    get_fields_for_table_one_field(relationship->get_to_table(), 
+  sharedptr<Field> to_key_field =
+    get_fields_for_table_one_field(relationship->get_to_table(),
       relationship->get_to_field());
   bool relationship_invalid = false;
   if(!to_key_field)
   {
-    Utils::show_ok_dialog(_("Invalid Relationship"), 
+    Utils::show_ok_dialog(_("Invalid Relationship"),
       _("The relationship may not be used to show related records because the relationship does not specify a field in the related table."),
      *this, Gtk::MESSAGE_ERROR);
     relationship_invalid = true;
   }
   else if(to_key_field->get_primary_key())
   {
-    Utils::show_ok_dialog(_("Relationship Uses a Related Primary Key"), 
+    Utils::show_ok_dialog(_("Relationship Uses a Related Primary Key"),
       _("The relationship may not be used to show related records because the relationship uses a primary key field in the related table, which must contain unique values. This would prevent the relationship from specifying multiple related records."),
      *this, Gtk::MESSAGE_ERROR);
     relationship_invalid = true;
   }
   else if(to_key_field->get_unique_key())
   {
-    Utils::show_ok_dialog(_("Relationship Uses a Related Unique Field"), 
+    Utils::show_ok_dialog(_("Relationship Uses a Related Unique Field"),
       _("The relationship may not be used to show related records because the relationship uses a unique-values field in the related table. This would prevent the relationship from specifying multiple related records."),
      *this, Gtk::MESSAGE_ERROR);
     relationship_invalid = true;
   }
-  
+
   //Reset the previous value if the choice was bad:
   if(relationship_invalid)
   {
     m_combo_relationship->set_selected_relationship(
-       m_portal->get_relationship(), m_portal->get_related_relationship()); 
+       m_portal->get_relationship(), m_portal->get_related_relationship());
     return;
   }
 
-  //Clear the list of fields if the relationship has changed, 
+  //Clear the list of fields if the relationship has changed,
   //because the fields could not possible be correct for the new table:
   bool relationship_changed = false;
   const Glib::ustring old_relationship_name = glom_get_sharedptr_name(m_portal->get_relationship());
@@ -400,6 +454,11 @@ void Dialog_Layout_List_Related::on_combo_relationship_changed()
   m_modified = true;
 }
 
+void Dialog_Layout_List_Related::on_spinbutton_changed()
+{
+  m_modified = true;
+}
+
 sharedptr<Relationship> Dialog_Layout_List_Related::get_relationship() const
 {
   std::cout << "debug: I wonder if this function is used." << std::endl;
@@ -424,7 +483,7 @@ void Dialog_Layout_List_Related::on_button_add_field()
   //std::cout << "debug table used =" << m_portal->get_table_used(m_table_name) << std::endl;
 
   type_list_field_items fields_list = offer_field_list(m_portal->get_table_used(m_table_name), this);
-  for(type_list_field_items::iterator iter_chosen = fields_list.begin(); iter_chosen != fields_list.end(); ++iter_chosen) 
+  for(type_list_field_items::iterator iter_chosen = fields_list.begin(); iter_chosen != fields_list.end(); ++iter_chosen)
   {
     sharedptr<LayoutItem_Field> field = *iter_chosen;
     if(!field)
@@ -492,4 +551,3 @@ Glib::ustring Dialog_Layout_List_Related::get_fields_table() const
 }
 
 } //namespace Glom
-
